#!/bin/bash
# +-----------------------------------------------------------+
# |                                                           |
# | use this script to generate some mode lines automatically |
# | with videogen.                                            |
# |                                                           |
# | (c) Szabolcs Rumi, 1997-2001                              |
# |                                                           |
# +-----------------------------------------------------------+

# you may change this to the location of the videogen program
VIDEOGEN=${VIDEOGEN=videogen}

# set this to the maximum dot clock your video card can generate [MHz]
DOTCLOCK="225"

# set this to the maximum horizontal refresh rate of your monitor [kHz]
HORIZ="64"

# set this to the maximum vertical refresh rate of your monitor [Hz]
VERT="120"

# set this to the minimum horizontal sync pulse length of your monitor [us]
MINSYNC="2"

# load our settings
. ~/.videogenrc

if test -z "$DIALOG" ; then
  which dialog   > /dev/null && DIALOG=dialog
  which whiptail > /dev/null && DIALOG=whiptail
  which gdialog > /dev/null  && DIALOG=gdialog
fi

# The dialog part
#DIALOG=${DIALOG=whiptail}
tempfile=`tempfile 2>/dev/null` || tempfile=/tmp/test$$
trap "rm -f $tempfile" 0 1 2 5 15

$DIALOG \
 --title "GENERAL WARNING" --clear        --yesno "WARNING! WARNING! WARNING!\n\nTHE USE OF THIS PROGRAM CAN CAUSE PERMANENT DAMAGE TO\nYOUR MONITOR AND/OR VIDEO CARD. IF YOU ARE NOT SURE\nWHAT YOU ARE DOING, HIT CANCEL NOW, OTHERWISE, HIT OK\nTO CONTINUE.\nTHE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF\nANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT\nLIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.\nIN NO EVENT SHALL THE SOFTWARE AUTHOR(s) BE LIABLE\nFOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER\nIN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\nFROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE\nOR THE USE OR OTHER DEALINGS IN THE SOFTWARE." 20 60|| exit 1

 
$DIALOG --title "HINTS" --clear --msgbox \
"There are no perfect modelines for all monitors. And there are many monitors that can be damaged by using wrong and non-standard modes, so better make sure that it will power-off if an invalid mode is tried. Also note that when the initial set of Modeline has been generated and you can run X, you can use xvidtune for fine configuration. Using xvindtune's 'auto' mode, you can find the best working timing values, than you can hit 'show' to dump them to the console and replace the values of the actuall Modeline with the dumped numbers." 20 60

$DIALOG \
 --title "INPUT BOX" --clear \
        --inputbox "Please enter the maximum dot clock your video card can generate [MHz]. (default: 225)" 10 51 $DOTCLOCK 2> $tempfile
retval=$?

case $retval in
  0)
    DOTCLOCK=`cat $tempfile`;;
  *)
    exit 1;;
esac

$DIALOG \
 --title "INPUT BOX" --clear \
        --inputbox "Please enter the maximum horizontal refresh rate of your monitor [kHz]. (default: 64)" 10 51 $HORIZ 2> $tempfile
retval=$?
case $retval in
  0)
    HORIZ=`cat $tempfile`;;
  *)
    exit 1;;
esac

$DIALOG \
 --title "INPUT BOX" --clear \
        --inputbox "Please enter the maximum vertical refresh rate of your monitor [Hz]. (default: 120)" 10 51 $VERT 2> $tempfile
retval=$?

case $retval in
  0)
    VERT=`cat $tempfile`;;
  *)
    exit 1;;
esac

$DIALOG \
 --title "INPUT BOX" --clear \
        --inputbox "Please enter the minimum horizontal sync pulse length of your monitor [us]. (default: 2)" 10 51 $MINSYNC 2> $tempfile
retval=$?

case $retval in
  0)
    MINSYNC=`cat $tempfile`;;
  *)
    exit 1;;
esac

if $DIALOG \
 --title "NVIDIA COMPATIBILITY" --yesno "The non-free Nvidia drivers available from www.nvidia.com require small changes in calculating algorithm. Without them, many Modelines are not accepted by the driver. Should videogen try to make compatible lines?" 10 51 2> $tempfile ; then
  VIDEOGEN="$VIDEOGEN -n"
fi

rm -rf ~/.videogenrc
(cat <<EOF
export DOTCLOCK="$DOTCLOCK"
export HORIZ="$HORIZ"
export VERT="$VERT"
export MINSYNC="$MINSYNC"
EOF
) | tee ~/.videogenrc

# these lines were inserted to warn you not to forget to change the above vars
# which are set to my hardware parameters by default
echo -n [1m
echo --------------------------------------------------------------------------
echo Now you can edit your XF86Config\(-4\) file, locate the \"Monitor\"
echo section and input the generated lines there. For framebuffer lines,
echo use fbset directly or enter them into /etc/fb.modes. Take a look at the
echo manpages fbset\(1\) and fb.modes\(5\) to see what this is about.
echo --------------------------------------------------------------------------
echo -n [0m

# these are the lines printing out the main thing
echo Modeline `$VIDEOGEN -s -width=640 -height=400 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo Modeline `$VIDEOGEN -s -width=640 -height=480 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo Modeline `$VIDEOGEN -s -width=800 -height=600 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo Modeline `$VIDEOGEN -s -width=1024 -height=768 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo Modeline `$VIDEOGEN -s -width=1152 -height=864 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo Modeline `$VIDEOGEN -s -width=1280 -height=960 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo Modeline `$VIDEOGEN -s -width=1280 -height=1024 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo Modeline `$VIDEOGEN -s -width=1600 -height=1200 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=640 -height=400 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=640 -height=480 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=800 -height=600 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=1024 -height=768 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=1152 -height=864 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=1280 -height=960 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=1280 -height=1024 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`
echo timings `$VIDEOGEN -s -fb -width=1600 -height=1200 -clk=$DOTCLOCK -hf=$HORIZ -vf=$VERT -hsync=$MINSYNC`

echo Press RETURN to continue...
read
# EOF
